/**
Licensed Materials - Property of IBM

IBM Cognos Products: DOCS

(C) Copyright IBM Corp. 2023

US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with
IBM Corp.
*/
/**
 *
 * Copyright (C) 2008 Cognos ULC, an IBM Company. All rights reserved.
 * Cognos (R) is a trademark of Cognos ULC, (formerly Cognos Incorporated).
 *
 */

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.*;

import javax.xml.namespace.QName;
import javax.xml.rpc.ServiceException;

import com.cognos.developer.schemas.bibus._3.*;

import org.apache.axis.client.Stub;
import org.apache.axis.message.SOAPHeaderElement;

/**
 *
 */
public class TriggerAPIKey
{
	private ContentManagerService_PortType cmService;
	private ContentManagerService_ServiceLocator cmServiceLocator;
	private final  EventManagementService_PortType eventService;
	private final  EventManagementService_ServiceLocator eventServiceLocator;

    private static final String SOAP_PATH = "p2pd/servlet/dispatch";
	private static final String REST_PATH = "api/v1/";
	private final URL soapURL;
	private final URL restURL;

	public TriggerAPIKey(final URL endpoint) throws MalformedURLException, ServiceException {
		//initialize the service locators
		eventServiceLocator = new EventManagementService_ServiceLocator();
		cmServiceLocator = new ContentManagerService_ServiceLocator();

		soapURL = new URL(endpoint, SOAP_PATH);
		// System.out.println("Using soap url: " +  soapURL);

		restURL = new URL(endpoint, REST_PATH);
		// System.out.println("Using rest api: " +  restURL);

		//get the service objects from the locators
		cmService = cmServiceLocator.getcontentManagerService(soapURL);
		eventService = eventServiceLocator.geteventManagementService(soapURL);

		//Set the axis timeout to 0 (infinite)
		//There may be many, many actions due to this trigger
		((Stub)eventService).setTimeout(0);
	}
	
	public boolean login(String apiKey) {
		  try {

			  final URL sessionURL = new URL(restURL, "session");
			  // System.out.println("Using session api at: " +  sessionURL);

			  HttpURLConnection httpURLConnection = (HttpURLConnection) sessionURL.openConnection();
			  httpURLConnection.setDoOutput(true);
			  httpURLConnection.setRequestMethod("PUT");
			  httpURLConnection.setRequestProperty("Content-Type", "application/json");
			  try ( OutputStreamWriter out = new OutputStreamWriter(httpURLConnection.getOutputStream())) {
				  out.write("{\"parameters\":[{\"name\":\"CAMAPILoginKey\",\"value\":\"" + apiKey + "\"}]}");
			  }

			  final int responseCode = httpURLConnection.getResponseCode();
			  if( responseCode < 200 || responseCode > 299) {
				  System.out.println("Login Failure. http code is: " + responseCode);
				  final InputStream errorStream = httpURLConnection.getErrorStream();
				  System.out.println("Server response:");
				  if( errorStream != null ) {
					  try (BufferedReader in = new BufferedReader(new InputStreamReader(errorStream))) {
						  in.lines().forEach(System.out::println);
					  }
				  }
				  return false;
			  }

			  // get the cookies and send them to CM to get a populated soap soap header.
			  final List<String> setCookies = httpURLConnection.getHeaderFields().get("Set-Cookie");
			  if( setCookies != null) {
				  final ArrayList<CookieVar> cookieVars = new ArrayList<>();
				  for (String cookie : setCookies) {
					  if (!cookie.contains("=")) continue;
					  if (cookie.contains(";")) {
						  cookie = cookie.split(";", 2)[0];
					  }
					  final String[] parts = cookie.split("=", 2);
					  final String name = parts[0];
					  final String value = parts[1];

					  cookieVars.add(new CookieVar(name, value));
				  }

				  final HdrSession hdrSession = new HdrSession();
				  hdrSession.setCookieVars(cookieVars.toArray(new CookieVar[0]));

				  BiBusHeader bibus = new BiBusHeader();
				  bibus.setHdrSession(hdrSession);
				  ((Stub)cmService).setHeader("http://developer.cognos.com/schemas/bibus/3/", "biBusHeader", bibus);
				  BaseClass[] result = cmService.query(
						  new SearchPathMultipleObject("~"),
						  new PropEnum[] {},
						  new Sort[] {},
						  new QueryOptions());

				  SOAPHeaderElement temp = ((Stub)cmService).getResponseHeader("http://developer.cognos.com/schemas/bibus/3/", "biBusHeader");
				  bibus = (BiBusHeader)temp.getValueAsType(new QName("http://developer.cognos.com/schemas/bibus/3/", "biBusHeader"));

				  if( result == null || bibus == null){
					  System.out.println("logon failed");
					  return false;
				  }
				  // System.out.println("Account search path:" + result[0].getSearchPath().getValue());

				  ((Stub)eventService).setHeader("http://developer.cognos.com/schemas/bibus/3/", "biBusHeader", bibus);

				  return true;

			  }
			  System.out.println("Login Failure, cookies not found. Response: ");
			  try (BufferedReader in = new BufferedReader(new InputStreamReader(httpURLConnection.getInputStream()))) {
				  in.lines().forEach(System.out::println);
			  }
			  return false;
		  } catch (Exception e){
			  e.printStackTrace();
			  return false;
		  }
	}

	public boolean logoff(){
		try {
			SOAPHeaderElement temp = ((Stub)eventService).getResponseHeader("http://developer.cognos.com/schemas/bibus/3/", "biBusHeader");
			BiBusHeader bibus = (BiBusHeader)temp.getValueAsType(new QName("http://developer.cognos.com/schemas/bibus/3/", "biBusHeader"));
			if (bibus != null)
			{
				((Stub)cmService).setHeader("http://developer.cognos.com/schemas/bibus/3/", "biBusHeader", bibus);
			}
			return true;
		}catch (Exception e){
			e.printStackTrace();
			return false;
		}
	}

	public int fireTrigger(String triggerName)
	{
		try
		{
			return eventService.trigger(triggerName);
		}
		catch(Exception e)
		{
			e.printStackTrace();
			return 0;
		}
	}

	public static void usage()
	{
		System.out.println("");
		System.out.println("");
		System.out.println("Command Line Parameters:");
		System.out.println("");
		System.out.println("<Server Base URL> <API Key> triggerList [ <delimiter> ]");
		System.out.println("");
		System.out.println("  Required arguments:");
		System.out.println("");
		System.out.println("      Server Base URL - note the required base URL format (no path) needed for hybrid use");
		System.out.println("        eg. \"http://localhost:9300/\"");
		System.out.println("");
		System.out.println("      API Key - user API key to to logon with. Must Renew Credentials before using new API keys for the first time.");
		System.out.println("");
		System.out.println("      triggerList - comma separated list of trigger names");
		System.out.println("        eg. \"triggerName1,triggerName2,triggerName3\"");
		System.out.println("");
		System.out.println("  NOTE: If using the optional delimiter override parameter defined below, use the delimiter specified in that parameter to separate the trigger names");
		System.out.println("			eg. \"triggerName1|triggerName2|triggerName3\"");
		System.out.println("");	
		System.out.println("  Optional argument:");
		System.out.println("");
		System.out.println("     delimiter - Use a different triggerList delimiter character to override the comma delimiter if trigger names contain commas");
		System.out.println("			eg. \"|\"");
		System.out.println("");
	}

	public static void main(String args[]) throws MalformedURLException, ServiceException {
		if ((args.length != 3) && // <API URL> <API Key> triggerList
			(args.length != 4))   // <API URL> <API Key> triggerList <delimiter>
		{
			usage();
			System.exit(-1);
		}

		final URL api_url = new URL(args[0].trim().endsWith("/") ? args[0] : args[0] + "/");
		final String api_key = args[1];
		final String triggers = args[2];
		final String delimiter = args.length == 4 ? args[3] : ",";

		TriggerAPIKey myTrigger = new TriggerAPIKey(api_url);

		if(myTrigger.login(api_key) == false){
			System.exit(-2);
		}

		StringTokenizer triggerTokens = new StringTokenizer(triggers, delimiter);

		int totalTriggersFired = 0;
		int triggerFired = 0;

		while(triggerTokens.hasMoreTokens())
		{
			String triggerName = triggerTokens.nextToken();
			triggerFired = myTrigger.fireTrigger(triggerName);
			if ( triggerFired > 0 )
			{
				System.out.println("Trigger: " + triggerName + " fired successfully");
				totalTriggersFired += 1;
			}
			else
			{
				System.out.println("");
				System.out.println("Failed to fire trigger " + triggerName + ".");
				System.out.println("");
				System.out.println("Note that if a schedule was not triggered, it may be ");
				System.out.println("because the current user does not have sufficient permissions ");
				System.out.println("to access the schedule. For more information, see the ");
				System.out.println("Administration and Security Guide.");
				System.out.println("");
			}
		}
		myTrigger.logoff();
		System.exit(totalTriggersFired);
	}
}
